package org.jtheque.core.managers.properties;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.ActivableManager;

import java.util.List;

/**
 * A properties manager specification.
 *
 * @author Baptiste Wicht
 */
public interface IPropertiesManager extends ActivableManager {
    /**
     * Return all the properties of a bean. For better performance, the results are internally cached.
     *
     * @param bean The bean to extract the properties from.
     * @return A List containing all the properties of the bean.
     * @throws PropertyException If an errors occurs during the recuperation of the properties.
     */
    List<Property> getProperties(Object bean) throws PropertyException;

    /**
     * Return the value of a property.
     *
     * @param bean     The bean to get the property value from.
     * @param property The property.
     * @return the value of the property.
     */
    Object getProperty(Object bean, Property property);

    /**
     * Return the value of a property.
     *
     * @param bean     The bean to get the property value from.
     * @param property The property.
     * @return the value of the property.
     */
    Object getProperty(Object bean, String property);

    /**
     * Create a memento for the bean. A memento is a clone of all the properties of the bean.
     *
     * @param bean The bean to create the memento from.
     * @param <T>  The class of the bean.
     * @return The memento.
     */
    <T> T createMemento(T bean);

    /**
     * Generate a toString() String based on all the properties of the bean.
     *
     * @param bean The bean.
     * @return A String representation of all the properties of the bean.
     */
    String toString(Object bean);

    /**
     * Generate a hashcode for the bean.
     *
     * @param bean The bean.
     * @return A hash code based on all the properties of the bean.
     */
    int hashCode(Object bean);

    /**
     * Restore the state of the memento. It seems to copy all the properties values from the memento to the bean.
     *
     * @param bean    The bean.
     * @param memento The memento.
     */
    void restoreMemento(Object bean, Object memento);
}