package org.jtheque.core.managers.view.impl.components.filthy.java2d;

import org.jtheque.core.managers.core.Core;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.view.able.components.ISplashView;
import org.jtheque.core.managers.resource.IResourceManager;
import org.jtheque.core.utils.ui.AnimationUtils;
import org.jtheque.utils.ui.PaintUtils;

import org.jdesktop.animation.timing.Animator;
import org.jdesktop.animation.timing.triggers.TimingTrigger;
import org.jdesktop.animation.timing.triggers.TimingTriggerEvent;

import javax.annotation.PostConstruct;

import java.awt.Graphics;
import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Component;
import java.awt.image.BufferedImage;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @author Baptiste Wicht
 */
public final class SplashScreenPane extends AlphaCapableComponent implements ISplashView {
    private boolean inited;

    private BufferedImage bImage;

    private Font fontName;
    private Font fontLoading;

    private final Point pImage;
    private final Point pTextName;
    private final Point pTextLoading;

    private final String[] loadings = {"Loading ", "Loading .", "Loading ..", "Loading ...", "Loading ..."};
    private int current;
    
    private static final int LOADING_FONT_SIZE = 20;
    private static final int TITLE_FONT_SIZE = 24;

    private static final int ANIMATION_DURATION = 2000;
    private Animator animation;

    /**
     * Construct a new SplashScreenPane.  
     */
    public SplashScreenPane() {
        super();

        pImage = new Point();
        pTextName = new Point();
        pTextLoading = new Point();
    }

    /**
     * Init the view. 
     */
    @PostConstruct
    public void init(){
        bImage = Managers.getManager(IResourceManager.class).getImage(Managers.getCore().getApplication().getPNGLogo());
    }
    
    @Override
    public void paint(Graphics g) {   
        Graphics2D g2 = (Graphics2D)g;

        PaintUtils.initHints(g2);

        if(!inited){
            initFonts(g2);

            computeSizes();

            inited = true;
        }

        Color color = new Color(255, 255, 255, (int)(getAlpha() * 255));

        paintName(g2, color);
        paintImage(g2);
        paintLoading(g2, color);
    }

    /**
     * Compute the sizes. 
     */
    private void computeSizes() {
        int nameWidth = getWidth(Core.getInstance().getApplication().getName(), fontName);
        int loadingWidth = getWidth("Loading ...", fontLoading);

        int height = getFontHeight(fontName) + 10 + bImage.getHeight() + getFontHeight(fontLoading);

        pTextName.y = (int)((getSize().getHeight() - height)  / 2);
        pImage.y = pTextName.y + getFontHeight(fontName) + 10;
        pTextLoading.y = pImage.y + bImage.getHeight() + 20;

        pTextName.x = (int)((getSize().getWidth() - nameWidth) / 2);
        pImage.x = (int)((getSize().getWidth() - bImage.getWidth()) / 2);
        pTextLoading.x = (int)((getSize().getWidth() - loadingWidth) / 2);
    }

    /**
     * Init the fonts. 
     * 
     * @param g2 The Graphics2D element. 
     */
    private void initFonts(Graphics g2) {
        fontName = g2.getFont().deriveFont(Font.BOLD, TITLE_FONT_SIZE);
        fontLoading = g2.getFont().deriveFont(Font.BOLD, LOADING_FONT_SIZE);
    }

    /**
     * Paint the loading text. 
     * 
     * @param g2 The Graphics2D element. 
     * @param color The color of the loading text. 
     */
    private void paintLoading(Graphics g2, Color color) {
        g2.translate(pTextLoading.x, pTextLoading.y);

        PaintUtils.drawString(g2, loadings[current], 0, 0, fontLoading, color);
    }

    /**
     * Paint the image. 
     * 
     * @param g2 The Graphics2D element. 
     */
    private void paintImage(Graphics2D g2) {
        g2.translate(pImage.x, pImage.y);

        PaintUtils.drawAlphaImage(g2, bImage, 0, 0, getAlpha());

        g2.translate(-pImage.x, -pImage.y);
    }

    /**
     * Paint the name of the application. 
     * 
     * @param g2 The Graphics2D element. 
     * @param color The color of the application name. 
     */
    private void paintName(Graphics g2, Color color) {
        g2.translate(pTextName.x, pTextName.y);

        PaintUtils.drawString(g2, Core.getInstance().getApplication().getName(), 0, 0, fontName, color);

        g2.translate(-pTextName.x, -pTextName.y);
    }

    /**
     * Return the font height. 
     * 
     * @param font The font height. 
     * 
     * @return the font height. 
     */
    private int getFontHeight(Font font) {
        return getFontMetrics(font).getHeight();
    }

    /**
     * Return the width of the text in the specified font. 
     * 
     * @param text The text to calculate the width. 
     * @param font The font. 
     * 
     * @return the width of the text. 
     */
    private int getWidth(String text, Font font){
        return getFontMetrics(font).stringWidth(text);
    }

    /**
     * Set the current loading index. 
     * 
     * @param current The current loading index. 
     */
    public void setCurrent(int current) {
        this.current = current;

        repaint();
    }

    /**
     * Return the current loading index. 
     * 
     * @return The current loading index. 
     */
    public int getCurrent() {
        return current;
    }

    @Override
    public void appearsAndAnimate() {
        Animator fadeIn =  AnimationUtils.createFadeInAnimator(this);

        animation = AnimationUtils.createLoopEffect(this, ANIMATION_DURATION, "current", 4);
        animation.setResolution(ANIMATION_DURATION / 4);

        TimingTrigger.addTrigger(fadeIn, animation, TimingTriggerEvent.STOP);

        fadeIn.start();
    }

    @Override
    public void disappear() {
        animation.pause();
        AnimationUtils.startFadeOut(this);
    }

    @Override
    public Component getImpl() {
        return this;
    }
}