package org.jtheque.core.managers.view.impl.frame;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.managers.message.Message;
import org.jtheque.core.managers.view.able.IMessageView;
import org.jtheque.core.managers.view.impl.frame.abstraction.SwingDialogView;
import org.jtheque.core.managers.view.impl.models.MessageModel;
import org.jtheque.core.managers.view.impl.actions.JThequeAction;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.utils.ui.GridBagUtils;

import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.annotation.PostConstruct;

import java.awt.Container;
import java.awt.Frame;
import java.util.List;

/**
 * @author Baptiste Wicht
 */
public final class MessageView extends SwingDialogView implements IMessageView {
    private static final long serialVersionUID = 6806683143749769795L;

    private final MessageModel model;
    private JLabel dateLabel;
    private JLabel sourceLabel;
    private JLabel titleLabel;
    private JTextArea messageArea;

    private JThequeAction closeAction;
    private JThequeAction displayNextAction;
    private JThequeAction displayPreviousAction;

    /**
     * Construct a new <code>MessageView</code>.
     *
     * @param view The parent frame.
     */
    public MessageView(Frame view) {
        super(view);

        model = new MessageModel();
    }

    @Override
    @PostConstruct
    public void build() {
        setTitleKey("messages.view.title");
        setContentPane(buildContentPane());
        setResizable(false);
        pack();

        SwingUtils.centerFrame(this);
    }

    /**
     * Init the contentpane and return it.
     *
     * @return The contentpane.
     */
    private Container buildContentPane() {
        PanelBuilder builder = new PanelBuilder();

        addLabels(builder);
        addFields(builder);

        builder.addButtonBar(builder.gbcSet(0, 4, GridBagUtils.HORIZONTAL, GridBagUtils.LINE_END, 2, 1), closeAction, displayNextAction, displayPreviousAction);

        return builder.getPanel();
    }

    /**
     * Add the labels to the view.
     * 
     * @param builder The builder to add the labels to. 
     */
    private static void addLabels(PanelBuilder builder) {
        builder.addInternationalizedLabel("messages.view.data.date", builder.gbcSet(0, 0, GridBagUtils.NONE, GridBagUtils.LINE_END), PanelBuilder.BOLD);
        builder.addInternationalizedLabel("messages.view.data.source", builder.gbcSet(0, 1, GridBagUtils.NONE, GridBagUtils.LINE_END), PanelBuilder.BOLD);
        builder.addInternationalizedLabel("messages.view.data.title", builder.gbcSet(0, 2, GridBagUtils.NONE, GridBagUtils.LINE_END), PanelBuilder.BOLD);
    }

    /**
     * Add the fields to the view.
     * 
     * @param builder The builder to add the fields to. 
     */
    private void addFields(PanelBuilder builder) {
        dateLabel = builder.add(new JLabel(model.isDefaultMessage() ? "" : model.getCurrentMessage().getDate().getStrDate()), 
                builder.gbcSet(1, 0));
        
        sourceLabel = builder.add(new JLabel(model.isDefaultMessage() ? "" : model.getCurrentMessage().getSource()), 
                builder.gbcSet(1, 1));
        
        titleLabel = builder.add(new JLabel(model.isDefaultMessage() ? "" : model.getCurrentMessage().getTitle()), 
                builder.gbcSet(1, 2));
        
        messageArea = new JTextArea(model.isDefaultMessage() ? "" : model.getCurrentMessage().getMessage());
        messageArea.setRows(8);
        messageArea.setEnabled(false);
        
        builder.addScrolled(messageArea, builder.gbcSet(0, 3, GridBagUtils.BOTH, GridBagUtils.LINE_START, 2, 1));
    }

    @Override
    public void next() {
        setMessage(model.getNextMessage());
    }

    @Override
    public void previous() {
        setMessage(model.getPreviousMessage());
    }

    /**
     * Set the message to display.
     *
     * @param message The message to display.
     */
    private void setMessage(Message message) {
        if(model.isDefaultMessage()){
            dateLabel.setText(message.getDate().getStrDate());
            sourceLabel.setText(message.getSource());
            titleLabel.setText(message.getTitle());
            messageArea.setText(message.getMessage());
        }
    }

    @Override
    protected void validate(List<JThequeError> errors) {
        //Nothing to validate
    }

    /**
     * Set the action to close the view. This is not for use, this is only for Spring Injection. 
     * 
     * @param closeAction The action. 
     */
    public void setCloseAction(JThequeAction closeAction) {
        this.closeAction = closeAction;
    }

    /**
     * Set the action to display the next message. This is not for use, this is only for Spring Injection. 
     * 
     * @param displayNextAction The action. 
     */
    public void setDisplayNextAction(JThequeAction displayNextAction) {
        this.displayNextAction = displayNextAction;
    }

    /**
     * Set the action to display the previous message. This is not for use, this is only for Spring Injection. 
     * 
     * @param displayPreviousAction The action. 
     */
    public void setDisplayPreviousAction(JThequeAction displayPreviousAction) {
        this.displayPreviousAction = displayPreviousAction;
    }
}